import {
  DatabaseErrorException,
  NotFoundException,
  RpcExceptionCode,
  RpcValidationPipe,
} from '@mobicoop/ddd-library';
import { Controller, UnauthorizedException, UsePipes } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { DeleteUsernameRequestDto } from './dtos/delete-username.request.dto';
import { DeleteUsernameCommand } from '@modules/authentication/core/application/commands/delete-username/delete-username.command';
import { GRPC_AUTHENTICATION_SERVICE_NAME } from '@src/app.constants';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class DeleteUsernameGrpcController {
  constructor(private readonly commandBus: CommandBus) {}

  @GrpcMethod(GRPC_AUTHENTICATION_SERVICE_NAME, 'DeleteUsername')
  async delete(data: DeleteUsernameRequestDto): Promise<void> {
    try {
      await this.commandBus.execute(new DeleteUsernameCommand(data));
    } catch (error: any) {
      if (error instanceof UnauthorizedException)
        throw new RpcException({
          code: RpcExceptionCode.PERMISSION_DENIED,
          message: error.message,
        });
      if (error instanceof NotFoundException)
        throw new RpcException({
          code: RpcExceptionCode.NOT_FOUND,
          message: error.message,
        });
      if (error instanceof DatabaseErrorException)
        throw new RpcException({
          code: RpcExceptionCode.INTERNAL,
          message: error.message,
        });
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: error.message,
      });
    }
  }
}
