import { Module, Provider } from '@nestjs/common';
import { CqrsModule } from '@nestjs/cqrs';
import { DecideGrpcController } from './interface/grpc-controllers/decide.grpc.controller';
import { DecisionQueryHandler } from './core/application/queries/decision/decision.query-handler';
import { DECISION_MAKER } from './authorization.di-tokens';
import { OpaDecisionMaker } from './infrastructure/opa.decision-maker';
import { HttpModule } from '@nestjs/axios';

const grpcControllers = [DecideGrpcController];

const queryHandlers: Provider[] = [DecisionQueryHandler];

const adapters: Provider[] = [
  {
    provide: DECISION_MAKER,
    useClass: OpaDecisionMaker,
  },
];

@Module({
  imports: [CqrsModule, HttpModule],
  controllers: [...grpcControllers],
  providers: [...queryHandlers, ...adapters],
  exports: [],
})
export class AuthorizationModule {}
